﻿using Bulldozer.DrawingObjects;
using Bulldozer.MovementStrategy;

namespace Bulldozer.Generics
{
    /// <summary>
    /// Класс для хранения коллекции
    /// </summary>
    internal class BulldozersGenericStorage
    {
        /// <summary>
        /// Словарь (хранилище)
        /// </summary>
        readonly Dictionary<string, BulldozersGenericCollection<DrawingBulldozer, DrawingObjectBulldozer>> _bulldozerStorages;
        /// <summary>
        /// Возвращение списка названий наборов
        /// </summary>
        public List<string> Keys => _bulldozerStorages.Keys.ToList();
        /// <summary>
        /// Ширина окна отрисовки
        /// </summary>
        private readonly int _pictureWidth;
        /// <summary>
        /// Высота окна отрисовки
        /// </summary>
        private readonly int _pictureHeight;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="pictureWidth"></param>
        /// <param name="pictureHeight"></param>
        public BulldozersGenericStorage(int pictureWidth, int pictureHeight)
        {
            _bulldozerStorages = new Dictionary<string, BulldozersGenericCollection<DrawingBulldozer, DrawingObjectBulldozer>>();
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
        }
        /// <summary>
        /// Добавление набора
        /// </summary>
        /// <param name="name">Название набора</param>
        public void AddSet(string name)
        {
            if (!_bulldozerStorages.ContainsKey(name))
            {
                _bulldozerStorages.Add(name, new BulldozersGenericCollection<DrawingBulldozer, DrawingObjectBulldozer>(_pictureWidth, _pictureHeight));
            }

        }

        /// <summary>
        /// Удаление набора
        /// </summary>
        /// <param name="name">Название набора</param>
        public void DelSet(string name)
        {
            // TODO: Прописать логику для удаления набора
            if (_bulldozerStorages.ContainsKey(name))
            {
                _bulldozerStorages.Remove(name);
            }

        }
        /// <summary>
        /// Доступ к набору
        /// </summary>
        /// <param name="ind"></param>
        /// <returns></returns>
        public BulldozersGenericCollection<DrawingBulldozer, DrawingObjectBulldozer>?
        this[string ind]
        {
            get
            {
                // TODO: Продумать логику получения набора
                if (_bulldozerStorages.ContainsKey(ind))
                {
                    return _bulldozerStorages[ind];
                }
                else
                {
                    return null;
                }
            }
        }
    }

}
